﻿using System;
using Android.App;
using Android.Content;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Android.OS;
using Android.Print;
using Android.Graphics;
using Android.Print.Pdf;
using System.IO;
using Android.Media;
using System.Collections.Generic;

namespace PrintingSupport
{
    [Activity (Label = "KitKatPrintDemo", Theme = "@android:style/Theme.Holo.Light")]
    public class PrintActivity : Activity
    {
        static internal net.cloudapp.pdfviewerexport.Service1 service;
        internal Dictionary<int, Bitmap> byteArrayDict = new Dictionary<int, Bitmap>();
        internal int TotalPage;
        internal int currentPage;

        protected override void OnCreate (Bundle bundle)
        {
            base.OnCreate (bundle);

            SetContentView (Resource.Layout.Print);

            service = new net.cloudapp.pdfviewerexport.Service1();
         
            bool result;
            service.LoadDocument("Manual.pdf", out TotalPage, out result);
            net.cloudapp.pdfviewerexport.MemoryStream stream = service.GetPage(0, true);
           
            currentPage = 0;
            Bitmap bm = BitmapFactory.DecodeByteArray(stream._buffer, 0, stream._buffer.Length);
            byteArrayDict.Add(0, bm);
            ImageView imageSource = FindViewById<ImageView>(Resource.Id.imageView1);
            imageSource.SetImageBitmap(byteArrayDict[currentPage]);

            var button = FindViewById<Button> (Resource.Id.button3);

            button.Click += (object sender, EventArgs e) => {

                var printManager = (PrintManager)GetSystemService (Context.PrintService);
                var content = FindViewById<ImageView>(Resource.Id.imageView1);
                var printAdapter = new GenericPrintAdapter (this,byteArrayDict[currentPage]);

                printManager.Print ("MyPrintJob", printAdapter, null);
            };

            var button1 = FindViewById<Button>(Resource.Id.button1);
            button1.Click += (object sender, EventArgs e) =>
            {

                currentPage--;
                if (currentPage < 0 || currentPage > TotalPage)
                {
                    currentPage++;
                    return;
                }
                if (!byteArrayDict.ContainsKey(currentPage))
                {
                    net.cloudapp.pdfviewerexport.MemoryStream temstream = service.GetPage(currentPage, true);
                    bm = BitmapFactory.DecodeByteArray(temstream._buffer, 0, temstream._buffer.Length);
                    byteArrayDict.Add(currentPage, bm);
                }
              
                imageSource.SetImageBitmap(byteArrayDict[currentPage]);
            };

            var button2 = FindViewById<Button>(Resource.Id.button2);

            button2.Click += (object sender, EventArgs e) =>
            {

                currentPage++;
                if (currentPage < 0 || currentPage > TotalPage)
                {
                    currentPage--;
                    return;
                }
                 if (!byteArrayDict.ContainsKey(currentPage))
                {
                    net.cloudapp.pdfviewerexport.MemoryStream temstream = service.GetPage(currentPage, true);
                    bm = BitmapFactory.DecodeByteArray(temstream._buffer, 0, temstream._buffer.Length);
                    byteArrayDict.Add(currentPage, bm);
                }
              
                imageSource.SetImageBitmap(byteArrayDict[currentPage]);
            };
        }
    }

    public class GenericPrintAdapter : PrintDocumentAdapter
    {
         Bitmap image;
        Context context;
        PrintedPdfDocument document;
        float scale;

        public GenericPrintAdapter (Context context, Bitmap view)
        {
            this.image = view;
            this.context = context;
        }

        public override void OnLayout (PrintAttributes oldAttributes, PrintAttributes newAttributes, 
                                       CancellationSignal cancellationSignal, LayoutResultCallback callback, Bundle extras)
        {
         
            document = new PrintedPdfDocument (context, newAttributes);
        
            CalculateScale (newAttributes);

            var printInfo = new PrintDocumentInfo
                .Builder("MyPrint2.pdf")
                .SetContentType(PrintContentType.Document)
                .SetPageCount(1)
                .Build();

            callback.OnLayoutFinished (printInfo, true);
        }
            
        void CalculateScale (PrintAttributes newAttributes)
        {
            int dpi = Math.Max (newAttributes.GetResolution ().HorizontalDpi, newAttributes.GetResolution ().VerticalDpi);

            int leftMargin = (int)(dpi * (float)newAttributes.MinMargins.LeftMils / 1000);
            int rightMargin = (int)(dpi * (float)newAttributes.MinMargins.RightMils / 1000);
            int topMargin = (int)(dpi * (float)newAttributes.MinMargins.TopMils / 1000);
            int bottomMargin = (int)(dpi * (float)newAttributes.MinMargins.BottomMils / 1000);

            int w = (int)(dpi * (float)newAttributes.GetMediaSize ().WidthMils / 1000) - leftMargin - rightMargin;
            int h = (int)(dpi * (float)newAttributes.GetMediaSize ().HeightMils / 1000) - topMargin - bottomMargin;

            scale = Math.Min ((float)document.PageContentRect.Width () / w, (float)document.PageContentRect.Height () / h);
        }

        public override void OnWrite (PageRange[] pages, ParcelFileDescriptor destination, 
                                      CancellationSignal cancellationSignal, WriteResultCallback callback)
        {
            PrintedPdfDocument.Page page = document.StartPage (0);
            page.Canvas.Scale(1-scale,1-scale);
            page.Canvas.DrawBitmap(image,0,0,new Paint());
            document.FinishPage (page);

            WritePrintedPdfDoc (destination);

            document.Close ();

            document.Dispose ();

            callback.OnWriteFinished (pages);
        }

        void WritePrintedPdfDoc (ParcelFileDescriptor destination)
        {
            var javaStream = new Java.IO.FileOutputStream (destination.FileDescriptor);
            var osi = new OutputStreamInvoker (javaStream);
            using (var mem = new MemoryStream ()) {
                document.WriteTo (mem);
                var bytes = mem.ToArray ();
                osi.Write (bytes, 0, bytes.Length);
            }
        }
    }
}


